"""COCO-style evaluation metrics.
COCO API: github.com/cocodataset/cocoapi/
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import json
import os
from absl import flags
from absl import logging

import numpy as np
from pycocotools.coco import COCO
from pycocotools.cocoeval import COCOeval

import tensorflow as tf

FLAGS = flags.FLAGS

class EvaluationMetric(object):
    def __init__(self, filename=None, include_mask=False):
        """Constructs COCO evaluation class.

        The _update_op() takes detections from each image and push them to
        self.detections. The _evaluate() loads a JSON file in COCO annotation format
        as the groundtruth and runs COCO evaluation.

        Args:
          filename: Ground truth JSON file name.
        """
        self._include_mask = include_mask

        if filename:
            self.coco_gt = COCO(filename)
        self.filename = filename
        self.metric_names = ['AP', 'AP50', 'AP75', 'APs', 'APm', 'APl', 'ARmax1',
                             'ARmax10', 'ARmax100', 'ARs', 'ARm', 'ARl']
        self._reset()

    # def _reset(self):
    #     """Reset COCO API object."""
    #     if self.filename is None:
    #         self.coco_gt = COCO()
    #     self.detections = []
    #     self.dataset = {
    #         'images': [],
    #         'annotations': [],
    #         'categories': []
    #     }
    #     self.image_id = 1
    #     self.annotation_id = 1
    #     self.category_ids = []

    def estimator_metric_fn(self, detections):
        """detections: Detection results in a tensor with each row representing
            [image_id, x, y, width, height, score, class]
          groundtruth_data: Groundtruth annotations in a tensor with each row
            representing [y1, x1, y2, x2, is_crowd, area, class].
        Returns:
          metrics_dict: A dictionary mapping from evaluation name to a tuple of
            operations (`metric_op`, `update_op`). `update_op` appends the
            detections for the metric to the `self.detections` list.
        """

        def _evaluate():
            """Evaluates with detections from all images with COCO API.

            Returns:
              coco_metric: float numpy array with shape [12] representing the
                coco-style evaluation metrics.
            """
                # Run on validation dataset.
            detections = np.array(self.detections)
            image_ids = list(set(detections[:, 0]))
            coco_dt = self.coco_gt.loadRes(detections)
            # iouType    - ['segm'] set iouType to 'segm', 'bbox' or 'keypoints'
            coco_eval = COCOeval(self.coco_gt, coco_dt, iouType='bbox')
            coco_eval.params.imgIds = image_ids
            coco_eval.evaluate()
            coco_eval.accumulate()
            coco_eval.summarize()
            coco_metrics = coco_eval.stats

            if self._include_mask:
                # iouType    - ['segm'] set iouType to 'segm', 'bbox' or 'keypoints'
                mcoco_eval = cocoeval.COCOeval(coco_gt, coco_dt, iouType='segm')
                mcoco_eval.params.imgIds = image_ids
                mcoco_eval.evaluate()
                mcoco_eval.accumulate()
                mcoco_eval.summarize()
                mask_coco_metrics = mcoco_eval.stats

            if self._include_mask:
                metrics = np.hstack((coco_metrics, mask_coco_metrics))
            else:
                metrics = coco_metrics

            # clean self.detections after evaluation is done.
            # this makes sure the next evaluation will start with an empty list of
            # self.detections.
            # self._reset()
            return np.array(metrics, dtype=np.float32)

        def _update_op(detections):
            """Update detection results and groundtruth data.

            Args:
             detections: Detection results in a tensor with each row representing
               [image_id, x, y, width, height, score, class].
            """
            for i in range(len(detections)):
                # Filter out detections with predicted class label = -1.
                indices = np.where(detections[i, :, -1] > -1)[0]
                detections[i] = detections[i, indices]
                if detections[i].shape[0] == 0:
                    continue
                # Add images.
                image_id = detections[i][0, 0]
                if image_id == -1:
                    image_id = self.image_id
                detections[i][:, 0] = image_id
                self.detections.extend(detections[i])

        with tf.name_scope('coco_metric'):
            update_op = tf.py_func(_update_op, [detections], [])
            metrics = tf.py_func(_evaluate, [], tf.float32)
            metrics_dict = {}
            for i, name in enumerate(self.metric_names):
                metrics_dict[name] = (metrics[i], update_op)
            return metrics_dict
